package com.google.gson.metrics;

import com.google.gson.stream.JsonReader;
import java.io.StringReader;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

public class UtilDeserialisationJson {

  private UtilDeserialisationJson() {
    throw new AssertionError("La classe UtilDeserialisationJson ne doit pas être instanciée.");
  }

  // Désérialise une liste d'objets de type T à partir d'une chaîne JSON.
  public static <T> List<T> deserialiserListe(String json, Class<T> clazz) throws Exception {
    JsonReader jr = new JsonReader(new StringReader(json));
    List<T> resultat = new ArrayList<>();
    jr.beginArray();
    while (jr.hasNext()) {
      resultat.add(deserialiserObjet(jr, clazz));
    }
    jr.endArray();
    return resultat;
  }

  // Désérialise un objet de type T à partir d'un JsonReader.
  public static <T> T deserialiserObjet(JsonReader jr, Class<T> clazz) throws Exception {
    T objet = clazz.getDeclaredConstructor().newInstance();
    jr.beginObject();
    while (jr.hasNext()) {
      String name = jr.nextName();
      Field champ = chercherChamp(clazz, name);
      if (champ != null) {
        affecterChamp(jr, objet, champ);
      }
    }
    jr.endObject();
    return objet;
  }

  // Cherche le champ correspondant dans la classe spécifiée par son nom.
  private static Field chercherChamp(Class<?> clazz, String nomChamp) {
    for (Field champ : clazz.getDeclaredFields()) {
      if (champ.getName().equals(nomChamp)) {
        return champ;
      }
    }
    return null;
  }

  // Affecte la valeur lue depuis le JsonReader au champ de l'objet.
  private static void affecterChamp(JsonReader jr, Object objet, Field champ) throws Exception {
    Class<?> typeChamp = champ.getType();
    champ.setAccessible(true); // Pour accéder aux champs privés
    if (typeChamp.equals(long.class)) {
      champ.setLong(objet, jr.nextLong());
    } else if (typeChamp.equals(int.class)) {
      champ.setInt(objet, jr.nextInt());
    } else if (typeChamp.equals(boolean.class)) {
      champ.setBoolean(objet, jr.nextBoolean());
    } else if (typeChamp.equals(String.class)) {
      champ.set(objet, jr.nextString());
    } else {
      throw new RuntimeException("Type inattendu: " + typeChamp + ", nom: " + champ.getName());
    }
  }
}
